/*
    FEARLUS model0-7-1ci: CautiousImitativeOptimumStrategy.h
    Copyright (C) 1999-2003, 2006  Macaulay Institute

    This file is part of FEARLUS model0-7-1ci, an agent-based model of land use
    change.

    FEARLUS model0-7-1ci is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-7-1ci is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/* 

Interface for the CautiousImitativeOptimumStrategy object. This is a
historical imitative strategy for determining the land use of a land
parcel. It makes use of the particular way in which the yield is
calculated to compute, for each neighbouring parcel, the expected
yield for the decision parcel had it used the same land use as the
neighbouring parcel. (Assuming 0 don't care bits, 0 neighbourhood
noise, and the same climate and economy in the following year.)  This
value is computed using the bits that differ between the neighbouring
parcel and the decision parcel, the yield of the neighbouring parcel,
and the number of bits in the land use bit string.



CautiousImitativeOptimisers (unlike ImitativeOptimisers) ignore
neighbourhood weighting. They also differ in computing a variance in
expected yield, and inspecting all land parcels in the social
neighbourhood, rather than, as the ImitativeOptimisers do, just
finding an exemplar for each land use. Theoretically, the expected
yield should be the same regardless of the variance, but this strategy
is cautious in not believing this. The score for each land use is set
to the expected yield calculated for the land parcel with the lowest
variance in expected yield for that land use. The strategy then
chooses the highest expected yield land use, choosing at random
between land uses with equal highest expected yield. Theoretically,
therefore, CautiousImitativeOptimisers should choose the same land use
as ImitativeOptimisers, given the same seed and initial
conditions. (And, of course, zero neighbourhood weighting...)

*/

#import <objectbase/SwarmObject.h>
#import "Strategy.h"

@interface CautiousImitativeOptimumStrategy: SwarmObject
<ImitativeStrategy, HistoricalStrategy> {
  LandManager *lm;
  Parameter *parameter;
}

@end
